#
# PROGRAM: PYTHON CONSOLE DATABASE BROWSER
# MODULE : MYSQL DATABASE CONNECTOR
#

import mysql.connector
import traceback
import sys
import os
import json
import curses
from pydbro.py_log import msg_log

connfile = "sqlmk_conn.json"

constr = None
dsn = None

def get_mysql_db():
    db=None
    if os.path.exists("sqlmk_conn.json"):
      f = open("sqlmk_conn.json", "r")
      dbcontmp = f.read()
      f.close()
      constr = json.loads(dbcontmp)
      #msg_log(str(constr))
      if "database" in constr: 
        db = constr["database"]
        #msg_log(db)
    return(db)

def create_con(con):
    #global con
    global constr
    if os.path.exists("sqlmk_conn.json"):
        f = open("sqlmk_conn.json", "r")
        dbcontmp = f.read()
        f.close()
        constr = json.loads(dbcontmp)
    else:
        curses.endwin()
        print("Please use coned to prepare connection")
        exit()
    # print(str(constr))
    if con is None:
      con = mysql.connector.connect(**constr)
    return(con)

def set_conn(p_db_name):
    global connstr
    connstr = p_db_name


def get_conn():
    global connstr
    return connstr


def qry2dict(con, qry, qry_params=()):
    global constr
    #global con
    res = {}
    cols = []
    try:
      if con is None:
        con = create_con(con)
      cur = con.cursor()
      cur.execute(qry, qry_params)
      if cur.description is None:
        con.commit()
      else:
        data = cur.fetchall()
        for col in cur.description:
          cols.append(col[0])
        #con.close()
        if len(data) > 0:
          i = 0
          for col in cols:
            res[col] = []
            for row in data:
              res[col].append(row[i])
            i += 1
      return(con,res,cols)
    except Exception as e:
      tramsg=""
      exc_type, exc_value, exc_traceback = sys.exc_info()
      traces = traceback.extract_tb(exc_traceback)
      for frame_summary in traces:
        tramsg+="{} {}".format(frame_summary.name, frame_summary.lineno)
      msg_log("ERROR : "+str(e)+" "+qry+" "+tramsg)
      pass



#
# PROGRAM: PYTHON CONSOLE DATABASE BROWSER
# MODULE : GET LIST VALUES EXCLUDING NONE
#

def get_list_vals(p_list):
  ret=[]
  for i in p_list:
    if i is not None:
      ret.append(len(str(i)))
  return(ret)



#
# PROGRAM: PYTHON CONSOLE DATABASE BROWSER
# MODULE : READ USER INPUT AS STRING
#

import curses

def read_str(screen, col, p_banner, strdef = "", passwd = 0):
  curses.curs_set(1)
  key = "0"
  if strdef != "":
    vstr = strdef
  else:
    vstr = ""
  screen.move(0,0)
  screen.addstr(p_banner,col["miGr"])
  #screen.move(1,0)
  #screen.addstr("-"*(wi-1),col["loGr"])
  screen.move(1,0)
  if len(vstr)>0:
    if passwd == 1:
      screen.addstr("*"*(len(vstr)),col["hiGr"])
    else:
      screen.addstr(vstr,col["hiGr"])
  while key != 10:
    #screen.clrtoeol()
    key = screen.getch()
    if (
         (key >= ord(" ") and key <= ord("~"))
      or (key == 127 or key == 263 or key == 8)
    ):
      strlen = len(vstr)
      if (key == 127 or key == 263 or key == 8):
        if strlen >= 0:
          vstr = vstr[:-1]
          strlen = len(vstr)
          screen.move(1,strlen)
          screen.delch()
      else:
        vstr += chr(key)
        screen.attrset(col["hiGr"])
        if passwd == 1:
          screen.addch("*")
        else:
          screen.addch(chr(key))
  curses.curs_set(0)
  return(vstr)





#
# PROGRAM: PYTHON CONSOLE DATABASE BROWSER
# MODULE : ORACLE DATABASE CONNECTOR
#

import cx_Oracle
import traceback
import sys
import os
import json
import curses
from pydbro.py_log import msg_log

connfile="sqlok_conn.json"

constr=None
dsn=None

def read_conn(connfile):
  global constr
  if os.path.exists(connfile):
    if constr is None:
      f = open(connfile, "r")
      dbcontmp = f.read()
      f.close()
      constr = json.loads(dbcontmp)
      msg_log("-- Reading connection file")
  else:
    curses.endwin()
    print("Please use coned to prepare connection")
    exit()

def create_dsn(con):
   read_conn(connfile)
   dsn = cx_Oracle.makedsn(
     host=constr["host"],
     port=constr["port"], 
     service_name=constr["database"]
   )
   if con is None:
     con = cx_Oracle.connect(
       user=constr["user"],
       password=constr["password"],
       dsn=dsn
     )
     msg_log("-- Ora Creating Con")
   return(con)

def process_data(cur,data):
  res={}
  cols=[]
  if cur.description is not None:
    for col in cur.description:
      cols.append(col[0])
  if len(data)>0:
    i=0
    for col in cols:
      res[col]=[]
      for row in data:
        res[col].append(row[i])
      i+=1
  return(res,cols)

def qry2dict(con,qry,qry_params=()):

  data = None
  res={}
  cols=[]

  if con is None:
    con = create_dsn(con)

  try:
    cur = con.cursor()
    try:
      cur.execute(qry,qry_params)
    except:
      if "select" in qry:
        msg_log("-- Trying to reconnect...")
        con = create_dsn(con)
        cur = con.cursor()
        cur.execute(qry,qry_params)
      else:
        msg_log("--! Problem executing Query")
        return(con,res,cols)
    try:
      data = cur.fetchall()
    except Exception as e:
      con.commit()
      return(con,res,cols)
    res,cols = process_data(cur,data)
    return(con,res,cols)
  except Exception as e:
    tramsg=""
    exc_type, exc_value, exc_traceback = sys.exc_info()
    traces = traceback.extract_tb(exc_traceback)
    for frame_summary in traces:
      tramsg+="{} {}".format(frame_summary.name, frame_summary.lineno)
    msg_log("ERROR : "+str(e)+" "+qry+" "+tramsg)
    pass



#
# PROGRAM: PYTHON CONSOLE DATABASE BROWSER
# MODULE : QUERY METHOD SETUP
#

import os
import sqlite3

from pydbro.py_sqlite_db import qry2dict as cfg_qry2dict

cfg_dbf = ":memory:"

def prep_cfg_storage():
  ccon = sqlite3.connect(cfg_dbf)
  ccon,cfg,cfgcol = cfg_qry2dict(
    ccon,
    "create table if not exists cfg (db text, tab text, key text, value text, primary key(db,tab,key))",
    (),cfg_dbf)
  return(ccon)

def cfg_get_value(ccon,params):
  # Restore Current Table settings BEGIN
  ccon,cfg,cfgcol = cfg_qry2dict(
    ccon,
    "select * from cfg where tab = ? and key = ?",
    params,cfg_dbf
  )
  if "value" in cfg:
    return(cfg["value"][0])
  else:
    return(None)

def cfg_set_value(ccon,params):
  # Store Current Table Settings
  ccon,cfg,cfgcol = cfg_qry2dict(
    ccon,
    """insert into cfg values (?,?,?,?)
       on conflict (db,tab,key)
       do update set value = ?""",
       params,cfg_dbf
  )    

def cfg_get_all(ccon):
  ccon,cfg,cfgcol = cfg_qry2dict(
    "select * from cfg",
     (),cfg_dbf,ccon
  )
  return(cfg)

def cfg_path():
  cfg_dbf = ""
  try:
    home = os.path.expanduser("~")
  except Exception as e:
    pass
  try:
    os.mkdir(home+os.sep+".config")
  except Exception as e:
    pass
  try:
    os.mkdir(home+os.sep+".config"+os.sep+"pydbro")
  except Exception as e:
    pass
  try:
    cfg_dbf = home+os.sep+".config"+os.sep+"pydbro"+os.sep+"cfg.db"
  except Exception as e:
    msg_log(str(e))
    cfg_dbf = "cfg.db"
    pass
  return(cfg_dbf)



#
# PROGRAM: PYTHON CONSOLE DATABASE BROWSER
# MODULE : SQLITE DATABASE CONNECTOR
#

import sqlite3
import traceback
import sys

from datetime import datetime
from pydbro.py_log import msg_log

connstr="./db/001.db"

def set_conn(p_db_name):
  global connstr
  connstr=p_db_name

def get_conn():
  global connstr
  return(connstr)

def qry2dict(con, qry, qry_params=(),dbfile=None):
  res={}
  try:
    dt = str(datetime.now())
    #msg_log(dt+" "+qry+"\n") 
    #if dbfile is None:
    if con is None:
      con = sqlite3.connect(get_conn())
    cur = con.cursor()
    cur.execute(qry,qry_params)
    try: 
      data = cur.fetchall()
    except:
      con.commit()
      return(con,res,cols)
    cols=[]
    if cur.description is not None:
      for col in cur.description:
        cols.append(col[0])
      #if dbfile is not None:
      con.commit()
      #else:
      #  con.close()
      if len(data)>0:
        i=0
        for col in cols:
          res[col]=[]
          for row in data:
            res[col].append(row[i])
          i+=1
    return(con,res,cols)
  except Exception as e:
    #con.close()
    #traceback.print_exc(file=sys.stdout)
    #print ("ERR [ FAILED QRY DATABASE ] 001000x0010 (1) : Cannot Query Database")
    tramsg=""
    exc_type, exc_value, exc_traceback = sys.exc_info()
    traces = traceback.extract_tb(exc_traceback)
    for frame_summary in traces:
      tramsg+="{} {}".format(frame_summary.name, frame_summary.lineno)
    msg_log("ERROR : "+str(e)+" "+qry+" "+tramsg)
    pass
